package info.textgrid.lab.collatex.ui.parts;

import info.textgrid.lab.collatex.model.CollationResults;

import java.io.StringWriter;

import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.ContentViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.part.IPageSite;
import org.neo4j.graphdb.Transaction;

import eu.interedition.collatex.simple.SimpleVariantGraphSerializer;

/**
 * Result page that displays the collation results as an XML document. Lots TODO
 * here.
 */
public class TEIResultPage extends AbstractResultPage {

	private Composite control;
	private StyledText styledText;

	/**
	 * Create the PageBookView Page.
	 */
	public TEIResultPage() {
	}

	/**
	 * Create contents of the PageBookView Page.
	 * @param parent
	 */
	@Override
	public void createControl(final Composite parent) {
		final Composite container = new Composite(parent, SWT.NULL);

		control = container;
		container.setLayout(new GridLayout(1, false));
		{
			styledText = new StyledText(container, SWT.BORDER);
			styledText.setEditable(false);
			styledText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
					true, 1, 1));
			styledText
			.setFont(JFaceResources.getFont(JFaceResources.TEXT_FONT));

			if (getCollationSetEditor() != null
					&& getCollationSetEditor().getCollationResults() != null)
				newCollationResults(getCollationSetEditor()
						.getCollationResults());

		}

	}

	@Override
	public Control getControl() {
		return control;
	}

	@Override
	public void init(final IPageSite pageSite) {
		super.init(pageSite);
		createActions();
		initializeToolBar();
		initializeMenu();
	}

	@Override
	public void setFocus() {
		// Set the focus
	}

	/**
	 * Create the actions.
	 */
	private void createActions() {
		// Create the actions
	}

	/**
	 * Initialize the toolbar.
	 */
	private void initializeToolBar() {
		final IToolBarManager toolbarManager = getSite().getActionBars()
				.getToolBarManager();
	}

	/**
	 * Initialize the menu.
	 */
	private void initializeMenu() {
		final IMenuManager menuManager = getSite().getActionBars().getMenuManager();
	}

	@Override
	public void newCollationResults(final CollationResults results) {
		if (styledText == null)
			return;

		final Transaction transaction = results.getVariantGraph().getDatabase()
				.beginTx();
		try {
			final SimpleVariantGraphSerializer serializer = new SimpleVariantGraphSerializer(
					results.getVariantGraph());
			final StringWriter stringWriter = new StringWriter();
			XMLStreamWriter xmlStreamWriter;
			try {
				xmlStreamWriter = XMLOutputFactory.newFactory()
						.createXMLStreamWriter(stringWriter);
				serializer.toTEI(xmlStreamWriter);
				transaction.failure(); // we don't want ranks in the db
				styledText.setText(stringWriter.toString());
			} catch (final XMLStreamException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (final FactoryConfigurationError e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} finally {
			transaction.finish();
		}
	}

	@Override
	protected ContentViewer getViewer() {
		// TODO Auto-generated method stub
		return null;
	}
}
