package info.textgrid.lab.collatex.ui.utils;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.ui.progress.UIJob;

/**
 * A job that will run in the background but can represent its results in the UI
 * thread.
 */
public abstract class JobWithUICleanup extends Job {

	{
		addJobChangeListener(new JobChangeAdapter() {

			@Override
			public void done(final IJobChangeEvent event) {
				new UIJob(getName()) {

					@Override
					public IStatus runInUIThread(final IProgressMonitor monitor) {
						return JobWithUICleanup.this.done(event, monitor);
					}
				}.schedule();
			}

		});
	}

	/**
	 * {@inheritDoc}
	 */
	public JobWithUICleanup(final String name) {
		super(name);
	}

	/**
	 * This method will be called when the job has finished. It is guaranteed to
	 * run in the UI thread.
	 * 
	 * @param event
	 *            the {@link IJobChangeEvent} that has triggered the call to
	 *            this method
	 * @param monitor
	 *            a monitor for the UI job. Implementers may ignore this
	 *            parameter if they want
	 */
	protected abstract IStatus done(IJobChangeEvent event,
			IProgressMonitor monitor);

}
